#!/bin/bash
 
# Copyright (c) 2022 xFusion Digital Technologies Co., Ltd. All rights reserved.
# Description: Get the mainboard UUID for authentication management
# Author: yangzhuangzhuang yangzhuangzhuang@xfusion.com
# Function:-c|--collect [file]         Collect the UUID of the node machine
#          -g|--gte-uuid               Obtain the local UUID code
# History: Create at 2022.4.14
#          modify at 2022.5.10 by yangzhuangzhuang
 
SELINUX_CFG_DIR=/etc/selinux
EXCLUDE_LIST=${SELINUX_CFG_DIR}/fixfiles_exclude_dirs
VERIFY_DIR=/etc/FusionOS_Verify
VERIFY_FILE_NAME=FusionOS-license
license_file=${VERIFY_DIR}/${VERIFY_FILE_NAME}
all_node_license_file=/root/all_license_info.csv
old_char=('\\' '\}' '\[' '\$' '`' '\"' '\~')
new_char=('\\\\' '\\\}' '\\\[' '\\\$' '`' '\\\"' '\\\~')
 
mkdir -p ${SELINUX_CFG_DIR}
mkdir -p ${VERIFY_DIR}
touch ${EXCLUDE_LIST}
 
#Check whether the package is installed
function check_rpm_install() {
    rpm_name=$1
    rpm -q ${rpm_name} >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        yum install ${rpm_name} -y >/dev/null 2>&1
        if [ $? -ne 0 ]; then            
	    echo "install ${rpm_name} command fail"
            exit 1
        fi
    fi
}

#Check whether it is powered on for the first time
#If it is not the first boot and the UUID has been obtained, the UUID number is printd and exit.
function first_boot() {
    if [ -f ${license_file} ]; then
        uuid=$(grep ^UUID ${license_file} | awk -F " " '{print $3}')
        echo "You have obtained the license information,UUID is ${uuid}"
        exit 0
    fi

}

#Getting the environment Type
function get_env_type() {
    check_rpm_install dmidecode
    type=$(dmidecode -s system-manufacturer)
}

#if is physical,querying the Number of sockets
function get_socket_num() {
    export LANG=en_US.UTF-8
    socket_num=$(lscpu | grep -i "socket(s)" | awk -F " " '{print $NF}')
}

#Get uuid
function get_uuid() {
    system_uuid=$(dmidecode -s system-uuid)
    system_uuid_trimmed=${system_uuid//-/}
    rootfs_uuid=$(lsblk -o uuid,mountpoint -x mountpoint | grep '/$')
    uuid="${system_uuid_trimmed:12:6}${rootfs_uuid:0:4}${system_uuid_trimmed:0:6}"
}

#Handle special character in password
function password_verify() {
    node_info_file=$1
    tmp_node_info_file=$(mktemp)
    cat ${node_info_file} | sed 1d | sed '/^$/d' >${tmp_node_info_file}
    for ((idx = 0; idx < ${#old_char[@]}; idx++)); do
        rule=s/${old_char[idx]}/${new_char[idx]}/g
        sed -i ${rule} $tmp_node_info_file >/dev/null 2>&1
    done
    node_info_file=$tmp_node_info_file
}

#Generate a license file that records the environment type, socket number, UUID, and whether the license is activated
function generate_license() {
    get_env_type
    get_socket_num
    get_uuid
    if [ -f ${license_file} ]; then
        chattr -ia ${license_file}
    fi
    echo "Type is ${type}" >${license_file}
    echo "Socket_num is ${socket_num}" >>${license_file}
    echo "UUID is ${uuid}" >>${license_file}
    # also copy to syslog
    echo "You have obtained the license information,UUID is ${uuid}"
    #change label and set ia attr
    if which chcon &>/dev/null; then
        chcon -t etc_t ${license_file} >/dev/null 2>&1 
    fi
    chattr +ia ${license_file}
    # modify ${EXCLUDE_LIST}:
    if ! grep "^\s*${VERIFY_DIR}\s*$" ${EXCLUDE_LIST} &>/dev/null; then
        echo "${VERIFY_DIR}" >>${EXCLUDE_LIST}
    fi
}

#Collect UUID information on a node.
function collect() {
    check_rpm_install expect
    check_rpm_install dos2unix
    check_rpm_install sqlite
    password_verify $1
    dbpath="${VERIFY_DIR}/FusionOS-license.db"
    export sqlitedb="sqlite3 $dbpath"
    $sqlitedb <<EOF
    create table if not exists license_info(date TEXT,Node_ip TEXT,UUID TEXT primary key,Env_type TEXT,Socket_num TEXT);
    insert into license_info values(datetime('now','localtime'),null,null,null,null);
.quit
EOF
    tmpfile=$(mktemp)
    while IFS= read -r line; do
	node_ip=$(echo ${line} | awk -F " " '{print $1}')
        ssh_user=$(echo ${line} | awk -F " " '{print $2}')
        ssh_pass=$(echo ${line} | awk -F " " '{print $3}')
        ping -c1 ${node_ip} >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            echo "${node_ip},the network is abnormal,will not collect."
            continue
        fi
        grep -w ${node_ip} /root/.ssh/known_hosts >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            expect -c "
                set timeout 2
                spawn ssh ${ssh_user}@${node_ip} \"cat /etc/FusionOS_Verify/FusionOS-license\"
                expect \"${ssh_user} * \"
                send \"${ssh_pass}\r\"
                expect eof" >${tmpfile}
        else
            expect -c "
                set timeout 2
                spawn ssh ${ssh_user}@${node_ip} \"cat /etc/FusionOS_Verify/FusionOS-license\"
                expect \"Are *\"
                send \"yes\r\"
                expect \"${ssh_user}@ *\"
                send \"${ssh_pass}\r\"
                expect eof" >${tmpfile}
        fi
        dos2unix ${tmpfile} >/dev/null 2>&1
        env_type=$(tail -n3 ${tmpfile} | grep ^Type | awk -F " " '{print $3}')
        socket_num=$(tail -n3 ${tmpfile} | grep ^Socket_num | awk -F " " '{print $3}')
        uuid=$(tail -n3 ${tmpfile} | grep ^UUID | awk -F " " '{print $3}')
        if [ "${uuid}" == "" ]; then
            echo -e "\"${node_ip} ${ssh_user} ${ssh_pass}\" : Failed to get data."
	    continue
        fi
        ${sqlitedb} >/dev/null 2>&1 <<EOF
        insert into license_info values(datetime('now','localtime'),'${node_ip}','${uuid}','${env_type}','${socket_num}');
.quit
EOF
    done < ${node_info_file}
    sqlite3 -csv -header ${dbpath} "select * from license_info" >${all_node_license_file}
    trap 'rm -f ${tmpfile} ${tmp_node_info_file}' EXIT
}

ARGS=$(getopt -o c:g --long collect:,get-uuid -n 'fusionos_idcollect.sh' -- "$@")
eval set -- "${ARGS}"
while true; do
    case "$1" in
        -c | --collect)
            case "$2" in
                "")
                    echo "option requires an argument"
                    shift 2
                    ;;
                *)
                    collect $2
                    shift 2
                    ;;
                esac
                ;;
        -g | --get-uuid)
            generate_license
            shift
            ;;
        --)
            shift
            break
            ;;
        *)
            echo "Invalid option"
            exit 1
            ;;
    esac
done

for arg in "$@"; do
    echo "processing $arg"
done

